﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of Bidder.

    Bidder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bidder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bidder; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[
    Functions for inter-addon communication with DKPmon
]]

local Comm_Version = 1.3

--[[
    Bidder:SendMessage
    Description:
        Combines sending to "GROUP" and "WHISPER" so that we receive our own messages.
    Input:
        p0...p15: Arguments to self:SendCommMessage("GROUP", ...)
    Output:
        None
]]
function Bidder:SendMessage(p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
    --self:Print("msging")
    local sent = self:SendCommMessage("GROUP", Comm_Version, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
    if sent == true then
        --self:Print("whispering")
--        self:SendCommMessage("WHISPER", UnitName("player"), Comm_Version, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
        if (DKPmon ~= nil) then 
            DKPmon:OnCommReceive(self.BidderCommPrefix, UnitName("player"), "GROUP", Comm_Version, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
        end
        Bidder:OnCommReceive(self.BidderCommPrefix, UnitName("player"), "GROUP", Comm_Version, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
    end
end

--[[
    Bidder:CommQueryPoints(console)
    Description:
        Send a query request for the user's current points total.
    Input:
        console: boolean, if true then this request is from a console command
    Output:
        None
]]
function Bidder:CommSendQueryPoints(console)
    local cmd = "QP"
    if console == true then
        cmd = "QC"
    end
    -- If we're sending a "QP" request, then send the message to the bidleader
    if cmd == "QP" then
        --self:SendCommMessage("WHISPER", self.bidleader, Comm_Version, cmd, self.db.realm.name)
        self:SendMessage(cmd, self.bidleader, self.db.realm.name)
    else
        self:SendMessage(cmd, self.db.realm.name)
    end
end

--[[
    Bidder:CommPlaceBids()
    Description:
        Sends the players item bids to DKPmon
    Input:
        None
    Output:
        None
]]
function Bidder:CommPlaceBids()
    -- Message grammar: 
    --   message = <itemString><itemString>+
    --   <itemString> = :<itemID>
    --   <itemID>     = [0-9]
    --     Note: the item id is simply the index of the item in the bidding list.
    local i=0
    local bidstring = ""
    while i ~= 10 do
        if self.GUI.bids[i].check:GetChecked() ~= nil then
            bidstring = string.format("%sY", bidstring)
        else
            bidstring = string.format("%sN", bidstring)
        end
        i = i + 1
    end
    -- Send the bid to the bid leader
    -- Message: "RB", bidname, senderclass, biddingclass, bidstring
    local myclassN = self.classInfo.classToNum[self.myclass]
    local bidclassN = self.classInfo.classToNum[self.db.realm.namelist[self.db.realm.name].class]
    --self:Print("bidding: "..Comm_Version.." myclassN = "..myclassN.." bidclassN = "..bidclassN)
    --self:SendCommMessage("WHISPER", self.bidleader, Comm_Version, "RB", self.db.realm.name, myclassN, bidclassN, bidstring)
    self:SendMessage("RB", self.bidleader, self.db.realm.name, myclassN, bidclassN, bidstring)
end

--[[
    Bidder:CommRecvBidItems(leader, items)
    Description:
        Called when DKPmon sends a list of items to bid on.
    Input:
        leader: The character that sent the list of biditems; this person is the bidding leader.
        items: table/list of strings. [<#>] = "<pool#>:<cost>:<colour>:<itemname>"
    Output:
        None
]]
function Bidder:CommRecvBidItems(leader, items)
    if self.items ~= nil then self:TableDelete(self.items) end
    self.items = {}
    self.bidleader = leader
    local i = 0
    local nitems = 0
    while i ~= 10 do
        if items[i] ~= nil then 
            -- Parse the string: items[i] = "<pool#>:<cost>:<colour>:<itemname>"
            local _,_,pool, cost, col, name = string.find(items[i], "(.+):(.+):(.+):(.+)")
            pool = tonumber(pool); cost = tonumber(cost)
            self.items[i] = { pool=pool, cost=cost, colour=col, name=name }
            nitems = nitems + 1
        end
        i = i + 1
    end
    -- Query the user's points
    self:CommSendQueryPoints(false, leader)
end

--[[
    Bidder:CommRecvPoints(msg)
    Description:
        Called when DKPmon replies to a points query
    Input:
        msg: string. Colon separated list of our points in each pool -- ":<pool0>:<pool1>:<pool2>:...:"
    Output:
        None
]]
function Bidder:CommRecvPoints(msg)
    self:TableDelete(self.mypoints)
    self.mypoints = {}

    -- Okay, let's parse that message.
    local pool = 0
--    for amount in string.gmatch(msg, ":([-]?[%d]*%.?[%d]+):") do
    -- Stopped working at some point. Fix by Dahn
    for amount in string.gmatch(msg, "([-]?[%d]*%.?[%d]+):") do
        self.mypoints[pool] = tonumber(amount)
        pool = pool + 1
    end
    self:OpenBidding()
end

--[[
    AceComm-2.0 callback
]]
function Bidder:OnCommReceive(prefix, sender, distribution, ver, a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11)
    --self:Print("Got message with prefix = "..prefix.." from "..sender.." on "..distribution.." of type "..a0)
    if prefix == self.DKPmonCommPrefix then
        -- Message from non-compatible DKPmon
        if ver ~= Comm_Version then return end

        -- Message from DKPmon
        --[[
            Okay, a0 contains the message command.
            "B" -- message is a list of items to bid on.
            "ACKB" -- Bidding acknowledgement message
            "RQP" -- message is a reply to a points query request
            "RQC" -- message is a reply to a points query request from console
            "C" -- close of bidding message.
            "QR" -- query for raid membership; reply with bidname iff we're in the same raid as the sender.
            "QS" -- query for standby membership; reply with bidname iff we're on standby and not in the same raid as the sender.
            "QI" -- installation check by DKPmon. 
        ]]
        if a0 == "B" then
            local items = {}
            if a1 ~= nil then items[0] = a1 end
            if a2 ~= nil then items[1] = a2 end
            if a3 ~= nil then items[2] = a3 end
            if a4 ~= nil then items[3] = a4 end
            if a5 ~= nil then items[4] = a5 end
            if a6 ~= nil then items[5] = a6 end
            if a7 ~= nil then items[6] = a7 end
            if a8 ~= nil then items[7] = a8 end
            if a9 ~= nil then items[8] = a9 end
            if a10 ~= nil then items[9] = a10 end
            self.biddingOpen = true
            self:CommRecvBidItems(sender, items)
            self:TableDelete(items)
            return
        end
        if a0 == "ACKB" then
            if (a1 ~= UnitName("player")) then return end
            self:Print(a2)
            return
        end
        if a0 == "RQP" then
            -- Is this message aimed at me?
            -- a1 = person this message is targetted at
            -- a2 = message to parse to get our point amounts
            if self.biddingOpen == false then return end -- Don't bother if bidding was closed
            if (a1 ~= UnitName("player")) then return end
            self:CommRecvPoints(a2)
            return
        end
        if a0 == "RQC" then
            if (a1 ~= UnitName("player")) then return end
            self:Print(a2)
            return
        end
        if a0 == "C" then
            self:ResetBidUI() -- Reset & close the bidding UI
            self.biddingOpen = false
            return
        end
        if a0 == "QR" then
            --self:Print("QR from "..sender)
            local inraid = self:NameInRaid(sender)
            if not inraid then return end -- sender's not in my raid
            --self:SendCommMessage("WHISPER", sender, Comm_Version, "RQR", self.classInfo.classToNum[self.myclass], self.db.realm.name, self.classInfo.classToNum[self.db.realm.namelist[self.db.realm.name].class])
            self:SendMessage("RQR", sender, self.classInfo.classToNum[self.myclass], self.db.realm.name, self.classInfo.classToNum[self.db.realm.namelist[self.db.realm.name].class])
            return
        end
        if a0 == "QS" then
            --self:Print("QS from "..sender)
            local inraid = self:NameInRaid(sender)
            if inraid then return end -- sender's in my raid, so I'm not on standby
            PlaySound("ReadyCheck")
            if self.onstandby == false then 
                self:Print("Standby check performed: You're not on standby")
                return
            end -- Not on standby, nothing to do.
            self:Print("Standby check performed: You're on standby")
            --self:Print("Sending QS reply")
            --self:SendCommMessage("WHISPER", sender, Comm_Version, "RQS", self.classInfo.classToNum[self.myclass], self.db.realm.name, self.classInfo.classToNum[self.db.realm.namelist[self.db.realm.name].class])
            self:SendMessage("RQS", sender, self.classInfo.classToNum[self.myclass], self.db.realm.name, self.classInfo.classToNum[self.db.realm.namelist[self.db.realm.name].class])
            return        
        end
        if a0 == "QI" then
            --self:SendCommMessage("WHISPER", sender, Comm_Version, "RQI")
            self:SendMessage("RQI", sender)
            return
        end
        return
    end
end