﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of Bidder.

    Bidder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bidder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bidder; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[
    Action commands for the GUI widgets
]]

local dewdrop = AceLibrary("Dewdrop-2.0")
local dewOptions = {}

--[[
    Bidder:PlaceBidsButtonPushed()
    Description:
        Called when the user pushes the "Place Bids" button on the GUI
        This function displays a confirmation dialog that will call
        the function to actually send the bids to DKPmon when the 'yes'
        button is pressed.
    Input:
        None
    Output:
        None
]]
function Bidder:PlaceBidsButtonPushed()
    if StaticPopupDialogs["BIDDER_PLACE_BID"] == nil then
        self:CreatePlaceBidDialog()
    end
    
    local nbids = self:CountPlacedBids()
    --if nbids == 0 then return end -- this should -NEVER- actually happen
    
    StaticPopup_Show("BIDDER_PLACE_BID", nbids)
end

--[[
    Bidder:NameSelectClick()
    Description:
        Called when the user clicks on their bidding name in the gui.
        Brings up a pulldown from which they can select which bidding name to use.
    Input:
        None
    Output:
        None
]]
function Bidder:NameSelectClick()
    self:TableDelete(dewOptions) -- Recursively reclaim the table
    dewOptions = { type = "group", args = {} }
    dewOptions.args.header = {} 
    dewOptions.args.header.type = "header"
    dewOptions.args.header.name = "Select the name to use for bidding"
    dewOptions.args.header.order = 1
    local name
    for name, _ in pairs(self.db.realm.namelist) do
        local n = name
        dewOptions.args[name] = {}
        local tab = dewOptions.args[name]
        tab.type = 'execute'
        tab.name = name
        tab.order = string.byte(name, 1)
        tab.desc = string.format("Set bidding name to %s", name)
        tab.func = function() 
                    Bidder:SetBiddingName(n)
                    dewdrop:Close()
                   end
    end
    dewdrop:Open(self.GUI.frame, 'children', dewOptions, 'point', "TOP", 'relativePoint', "TOP")
end