﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of Bidder.

    Bidder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bidder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bidder; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[
    GUI construction and manipulation functions
    
    Data fields for this UI are:
    
    Bidder.GUI.frame -- The bidding interface frame
    Bidder.GUI.clickFrame -- The frame under the bidding name.
    Bidder.GUI.name  -- Text string containing the bidding name
    Bidder.GUI.bidButton -- Button for sending the bids to DKPmon
    Bidder.GUI.bids -- Table of 10 widget-tables for each bid item
            [0..9] = { item -- Text field for the item name
                       pts  -- Text field points
                       check -- CheckButton for whether to bid on the item or not
                     }
]]


local function CreateBidLine(parent, line)
    local tab = {}
    local b,t
    local spacing=25
    -- Create the FontString for the item name
    t = parent:CreateFontString()
    t:SetPoint("TOPLEFT", parent, "TOPLEFT", 10, -80-line*spacing)
    t:SetFontObject(GameFontNormal); t:SetWidth(300)
    t:SetText("1234567890123456789012345678901234567890")
    tab.item = t

    -- Create the FontString for the points cost/total
    t = parent:CreateFontString()
    t:SetPoint("TOPLEFT", parent, "TOPLEFT", 310, -80-line*spacing)
    t:SetFontObject(GameFontNormal); t:SetWidth(80)
    t:SetText("9999/99999")
    tab.pts = t;
    
    -- Create the checkbox for whether to bid or not
    b = CreateFrame("CheckButton", nil, parent, "UICheckButtonTemplate")
    b:SetPoint("CENTER", parent, "TOPLEFT", 430, -80-line*spacing-5)
    b:SetChecked(false)
    b:SetScript("OnClick", function() Bidder:CheckClicked() end)
    tab.check = b
    
    tab.item:Hide()
    tab.pts:Hide()
    tab.check:Hide()
    return tab
end

--[[
    Bidder:CreateBidFrame()
    Description:
    Input:
        None
    Output:
        None
]]
function Bidder:CreateBidFrame()
    self.GUI = {}
    self.GUI.nbids = 0
    local frame = CreateFrame("Frame", "BidderFrame", UIParent)
    frame:SetWidth(480)
    frame:SetHeight(360)
    frame:SetBackdrop( {
            bgFile = "Interface/Tooltips/UI-Tooltip-Background",
            edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
            tile = true, tileSize=16, edgeSize=16,
            insets = {left=4, right=4, top=4, bottom=4}
        } )
    frame:SetBackdropColor(0.16, 0.28, 0.61, 0.9) -- Blueish
    --frame:SetBackdropColor(0.29, 0.72, 0.70, 0.9) --Cyan
    --frame:SetBackdropColor(0.89, 0.15, 0.22, 0.9) -- Red
    frame:EnableMouse(true)
    frame:SetMovable(true)
    frame:SetPoint("CENTER", 0, 0)
    frame:CreateTitleRegion()
    local tR= frame:GetTitleRegion()
    tR:SetWidth(150); tR:SetHeight(20)
    tR:SetPoint("CENTER", frame, "TOP", 0, -10)

    -- Add the title to the top of the window
    local t = frame:CreateFontString()
    t:SetAllPoints(tR)
    t:SetTextColor(1.0, 0.819, 0.0);
    t:SetWidth(140)
    t:SetFontObject(QuestTitleFont)
    t:SetText("Bidder")
    
    -- Add the "close window" button
    local b = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
    b:SetPoint("CENTER", frame, "TOPRIGHT", -16, -16)
    b:Show()
    
    -- Create the button to place bids.
    b = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
    b:SetWidth(150); b:SetHeight(22)
    b:SetText("Place Bids")
    b:SetPoint("TOPLEFT", frame, "TOPLEFT", 150, -330)
    b:SetScript("OnClick", function() Bidder:PlaceBidsButtonPushed() end)
    b:Disable()
    self.GUI.bidButton = b
    
    -- Create the text strings for the bidder's name
    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 20,-30)
    t:SetFontObject(GameFontWhite); t:SetHeight(16); t:SetWidth(300)
    t:SetText("Bidding name (click name to set):  ")
    
    -- Clickable frame at the bidding name)
    local f = CreateFrame("Frame", nil, frame)
    f:SetWidth(120); f:SetHeight(16);
    f:EnableMouse(true)
    f:SetPoint("TOPLEFT", frame, "TOPLEFT", 340,-30)
    f:SetScript("OnMouseDown", function() Bidder:NameSelectClick() end)
    f:Show()
    self.GUI.clickFrame = f

    t = f:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 340,-30)
    t:SetFontObject(GameFontGreen); t:SetHeight(16); t:SetWidth(120)
    t:SetJustifyH("LEFT")
    t:SetText("") --self.db.realm.name)
    self.GUI.name = t
    
    -- Column label strings
    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 10,-60)
    t:SetFontObject(GameFontWhite); t:SetWidth(300)
    t:SetText("Items")

    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 310,-60)
    t:SetFontObject(GameFontWhite); t:SetWidth(80)
    t:SetText("Points")

    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 390,-60)
    t:SetFontObject(GameFontWhite); t:SetWidth(80)
    t:SetText("Place Bid?")
    
        -- Create the 10 item bid lines.
    self.GUI.bids = {
        [0]=CreateBidLine(frame, 0),
        [1]=CreateBidLine(frame, 1),
        [2]=CreateBidLine(frame, 2),
        [3]=CreateBidLine(frame, 3),
        [4]=CreateBidLine(frame, 4),
        [5]=CreateBidLine(frame, 5),
        [6]=CreateBidLine(frame, 6),
        [7]=CreateBidLine(frame, 7),
        [8]=CreateBidLine(frame, 8),
        [9]=CreateBidLine(frame, 9)        
    }
    
    local dewdrop = AceLibrary("Dewdrop-2.0")
    dewdrop:Register(frame, { "dontHook", true })
    
--[[
    -- Draw the measuring stick
    local x = 10
    while x ~= 480 do
        t = frame:CreateFontString()
        t:SetPoint("TOPLEFT", frame, "TOPLEFT", x,-80)
        t:SetFontObject(GameFontWhite); t:SetWidth(5)
        t:SetJustifyH("LEFT")
        if math.mod(x, 20) == 0 then
            t:SetText("|")
        else
            t:SetText(".")
        end
        x = x + 10
    end
]]
    self.GUI.frame = frame
    self:ResetBidUI()
    frame:Hide()
end

function Bidder:CreatePlaceBidDialog()
    StaticPopupDialogs["BIDDER_PLACE_BID"] = {
        text = "Are you sure you want to bid on these %d items?",
        button1 = "Yes",
        button2 = "No",
        OnAccept = function()
            Bidder:CommPlaceBids()
        end,
        OnCancel = function() end,
        sound = "levelup2",
        timeout = 10,
        whiledead = 1,
        hideOnEscape = 1
    }
end

function Bidder:ShowBidUI()
    if self.GUI.frame == nil then self:CreateBidFrame(); end
    if self.biddingOpen == false then
        -- Disable the "Place Bids" button
        self.GUI.bidButton:Disable()
    else
        self.GUI.bidButton:Enable()
    end
    self.GUI.frame:Show()
end

function Bidder:CountPlacedBids()
    local b=0;
    local nbids = 0
    while b ~= 10 do
        if self.GUI.bids[b].check:GetChecked() ~= nil then
            nbids = nbids + 1
        end
        b = b + 1
    end
    return nbids
end

function Bidder:CheckClicked()
    -- Scan the list of checkboxes, if any of them are checked 
    -- then enable the place bids button. Else, disable it.
    --[[
    if self:CountPlacedBids() == 0 then
        self.GUI.bidButton:Disable()
    else
        self.GUI.bidButton:Enable()
    end
    ]]
end


function Bidder:ResetBidUI()
    if self.GUI.frame == nil then return end
    self.GUI.name:SetText(self.db.realm.name)
    self.GUI.bidButton:Disable()
    local i = 0
    while i ~= 10 do
        local tab = self.GUI.bids[i]
        
        tab.item:SetText("")
        tab.pts:SetText("/")
        tab.item:Hide()
        tab.pts:Hide()
        tab.check:SetChecked(false)
        tab.check:Hide()
        i = i + 1
    end
    self.GUI.frame:Hide()
end


--[[
    Bidder:OpenBidding()
    Description:
        Called when we've received both a list of items to bid on
        and a list of the user's points.
    Input:
        None
    Output:
        None
]]
function Bidder:OpenBidding()
    if self.GUI.frame == nil then self:CreateBidFrame(); end
    --self:ResetBidUI()
    self:ShowBidUI()
    local i=0
    while self.items[i] ~= nil do
       local tab = self.GUI.bids[i]
       tab.item:SetText(string.format("%d: |c%s[%s]|r", i, self.items[i].colour, self.items[i].name))
       tab.item:Show()
       tab.pts:SetText(string.format("%g/%g", self.items[i].cost, self.mypoints[self.items[i].pool]))
       tab.pts:Show()
       --tab.check:SetChecked(false)
       tab.check:Show()
       i = i + 1
    end
end
