﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of Bidder.

    Bidder is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bidder is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bidder; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]

Bidder = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceComm-2.0")

local dbversion = 1.1

--[[
    Initialize the database. 
    Database entries are:
        BiddingName: Name with which to place bids.
]]
Bidder:RegisterDB("BidderDB")
Bidder:RegisterDefaults('realm', {
    namelist = {},
    name = nil,
    dbversion = 0
    } )

function Bidder:OnInitialize()
    -- Called when the addon is loaded
    self.GUI = {}
    -- Perform out-of-date database check
    if self.db.realm.dbversion < dbversion then
        self:Print("Your database is out of date, resetting bidname")
        self.db.realm.name = nil
    end
    self.db.realm.dbversion = dbversion
    -- Determine whether this character is in the namelist yet.
    --  If it's not, add it.
    local charname, _ = UnitName("player")
    local _, classname = UnitClass("player")
    if self.db.realm.namelist[charname] == nil then

        self.db.realm.namelist[charname] = {}
        self.db.realm.namelist[charname].class = classname 
    end
    -- Set the initial bidding name to the character's name
    self:SetBiddingName(charname)
    self.myclass = classname
    
    --self.db.realm.name = charname
    
    -- Message prefix to use in inter-addon communication
    self.BidderCommPrefix = "Bidder"
    self.DKPmonCommPrefix = "DKPmon"

    self.CommPrefix = self.BidderCommPrefix
end

function Bidder:OnEnable()
    -- Called when the addon is enabled.
    self:Print("Enabled")
    -- Register for communication
    self:SetCommPrefix(self.CommPrefix)
    -- Register to receive messages from DKPmon
    self:RegisterComm(self.DKPmonCommPrefix, "GROUP")
    self:RegisterComm(self.DKPmonCommPrefix, "WHISPER")
    self:RegisterComm(self.DKPmonCommPrefix, "GUILD")
    -- Initialize the list of bidable items to empty
    self.items = {}
    self.mypoints = {}
    self.biddingOpen = false
    self.onstandby = false
end

function Bidder:OnDisable()
    -- Called when the addon is disabled.
    self:Print("Disabled")
    -- Unregister from communication
    --self:UnregisterAllComms() -- Note: automatically done
end

--[[ AceDB-2.0 Methods ]]
function Bidder:OnProfileDisable()
    -- Called when the user changes profiles (before change)
end

function Bidder:OnProfileEnable()
    -- Called when the user changes profiles (after change)
    self:SetBiddingName(self.db.realm.name)
end
